@@ -7,10 +7,14 @@
 
 import bibtexparser
 import collections.abc
+try:
+    import importlib.metadata as implib
+except Exception:
+    import importlib_metadata as implib
 import jinja2
 import json
 import logging
-import pkg_resources
+import pprint
 import reference_handler
 import seamm
 import seamm_util  # MUST come after seamm
@@ -27,14 +31,21 @@ job = printing.getPrinter()
 
 class Node(collections.abc.Hashable):
 
-    def __init__(self, flowchart=None, title='', extension=None, module=None):
+    def __init__(
+        self,
+        flowchart=None,
+        title='',
+        extension=None,
+        module=None,
+        logger=logger
+    ):
         """Initialize a node
 
         Keyword arguments:
         """
 
         self._uuid = uuid.uuid4().int
-        self.module = module
+        self.logger = logger
         self.parent = None
         self.flowchart = flowchart
         self._title = title
@@ -45,6 +56,9 @@ class Node(collections.abc.Hashable):
         self._references = None
         self._jinja_env = None
         self._graphs = None
+        self._options = {}  # Command-line options for this step
+        self._global_options = {}  # Command-line global options
+        self._step_type = None
 
         self.parameters = None  # Object containing control parameters
 
@@ -57,19 +71,24 @@ class Node(collections.abc.Hashable):
         self.formatter = logging.Formatter(fmt='{message:s}', style='{')
 
         # Setup the bibliography
-        self.bibliography = {}
-        if self.module:
-            filepath = pkg_resources.resource_filename(
-                self.module, 'data/references.bib'
-            )
-            logger.info("bibliography file path = '{}'".format(filepath))
+        self._bibliography = {}
+        package = self.__module__.split('.')[0]
+        files = [
+            p for p in implib.files(package) if 'references.bib' in str(p)
+        ]
+        if len(files) > 0:
+            path = files[0]
+            self.logger.info(f"bibliography file path = '{path}'")
 
-            if os.path.exists(filepath):
-                with open(filepath) as fd:
-                    tmp = bibtexparser.load(fd).entries_dict
-                writer = bibtexparser.bwriter.BibTexWriter()
-                for key, data in tmp.items():
-                    self.bibliography[key] = writer._entry_to_bibtex(data)
+            data = path.read_text()
+            tmp = bibtexparser.loads(data).entries_dict
+            writer = bibtexparser.bwriter.BibTexWriter()
+            for key, data in tmp.items():
+                self.logger.info(f'      {key}')
+                self._bibliography[key] = writer._entry_to_bibtex(data)
+            self.logger.debug(
+                'Bibliography\n' + pprint.pformat(self._bibliography)
+            )
 
     def __hash__(self):
         """Make iterable!"""
@@ -110,6 +129,28 @@ class Node(collections.abc.Hashable):
         """Whether this node has been visited in a traversal"""
         return self._visited
 
+    @property
+    def step_type(self):
+        """The step type, e.g. 'lammps-step', used for e.g. options"""
+        if self._step_type is None:
+            name = self.__module__.split('.')[0].replace('_', '-')
+            if name == 'seamm':
+                name = self.__module__.split('.')[1].replace('_', '-')
+                name += '-step'
+            self._step_type = name
+
+        return self._step_type
+
+    @property
+    def options(self):
+        """Dictionary of options for this step"""
+        return self._options
+
+    @property
+    def global_options(self):
+        """Dictionary of global options"""
+        return self._global_options
+
     @visited.setter
     def visited(self, value):
         self._visited = bool(value)
@@ -162,6 +203,22 @@ class Node(collections.abc.Hashable):
 
         self._references = value
 
+    @staticmethod
+    def is_expr(value):
+        """Return whether the value is an expression or constant.
+
+        Parameters
+        ----------
+        value : str
+            The value to test
+
+        Returns
+        -------
+        bool
+           True for an expression, False otherwise.
+        """
+        return len(value) > 0 and value[0] == '$'
+
     def set_uuid(self):
         self._uuid = uuid.uuid4().int
 
@@ -266,9 +323,9 @@ class Node(collections.abc.Hashable):
 
         next_node = self.next()
         if next_node:
-            logger.debug('returning next_node: {}'.format(next_node))
+            self.logger.debug('returning next_node: {}'.format(next_node))
         else:
-            logger.debug('returning next_node: None')
+            self.logger.debug('returning next_node: None')
 
         return next_node
 
@@ -278,10 +335,10 @@ class Node(collections.abc.Hashable):
 
         for edge in self.flowchart.edges(self, direction='out'):
             if edge.edge_subtype == 'next':
-                logger.debug('Next node is: {}'.format(edge.node2))
+                self.logger.debug('Next node is: {}'.format(edge.node2))
                 return edge.node2
 
-        logger.debug('Reached the end of the flowchart')
+        self.logger.debug('Reached the end of the flowchart')
         return None
 
     def previous(self):
@@ -314,18 +371,24 @@ class Node(collections.abc.Hashable):
         }
         data['attributes'] = {}
         for key in self.__dict__:
-            if key == 'flowchart':
+            # Remove unneeded variables
+            if key[0] == '_' and key not in ('_uuid', '_method', '_title'):
+                # _method needed because forcefield_step/forcefield.py does not
+                # use parameters yet!
                 continue
-            if key == 'parent':
-                continue
-            if key == 'formatter':
-                continue
-            if key == 'parser':
-                continue
-            if key == 'options':
-                continue
-            if key == 'unknown':
+            if key in (
+                'bibliography',
+                'flowchart',
+                'formatter',
+                'logger',
+                'options',
+                'parent',
+                'parser',
+                'tmp_table',
+                'unknown'
+            ):  # yapf: disable
                 continue
+
             if 'flowchart' in key:
                 # Have a subflowchart!
                 data[key] = self.__dict__[key].to_dict()
@@ -364,7 +427,9 @@ class Node(collections.abc.Hashable):
         # how many outgoing edges are there?
         n_edges = len(self.flowchart.edges(self, direction='out'))
 
-        logger.debug('node.default_edge_subtype, n_edges = {}'.format(n_edges))
+        self.logger.debug(
+            'node.default_edge_subtype, n_edges = {}'.format(n_edges)
+        )
 
         if n_edges == 0:
             return ""
@@ -558,16 +623,16 @@ class Node(collections.abc.Hashable):
             # current package first, then looks in the main SEAMM
             # templates.
             if module_path is None:
-                logger.info("Reading graph templates from 'seamm'")
+                self.logger.info("Reading graph templates from 'seamm'")
                 loaders = [jinja2.PackageLoader('seamm')]
             else:
-                logger.info(
+                self.logger.info(
                     "Reading graph templates from the following modules, "
                     "in order"
                 )
                 loaders = []
                 for module in module_path:
-                    logger.info('\t' + module)
+                    self.logger.info('\t' + module)
                     loaders.append(jinja2.PackageLoader(module))
 
             self._jinja_env = jinja2.Environment(
@@ -578,3 +643,52 @@ class Node(collections.abc.Hashable):
             jinja_env=self._jinja_env, template=template, title=title
         )
         return figure
+
+    def create_parser(self, name=None):
+        """Create the parser for this node.
+
+        All nodes have at least --log-level, which is setup here,
+
+        Parameters
+        ----------
+        name : str
+            The name of the parser. Defaults to a name derived from the module.
+
+        Returns
+        -------
+        seamm.Node()
+            The next node in the flowchart.
+        """
+        if self.visited:
+            return None
+
+        self.visited = True
+        result = self.next()
+
+        step_type = self.step_type
+        parser = seamm.getParser()
+
+        if not parser.exists(step_type):
+            parser.add_parser(step_type)
+
+            parser.add_argument_group(
+                step_type, 'debugging options',
+                'Options for turning on debugging output and tools'
+            )
+
+            parser.add_argument(
+                step_type,
+                '--log-level',
+                group='debugging options',
+                default='WARNING',
+                type=str.upper,
+                choices=[
+                    'NOTSET', 'DEBUG', 'INFO', 'WARNING', 'ERROR', 'CRITICAL'
+                ],
+                help=(
+                    "The level of informational output, defaults to "
+                    "'%(default)s'"
+                )
+            )
+
+        return result
