@@ -58,6 +58,8 @@ class TkJobHandler(object):
         """The list of dashboards."""
         result = []
         for dashboard in self.config:
+            if dashboard == 'global options':
+                continue
             if (
                 dashboard
                 not in ('global options', self.config.default_section)
@@ -302,8 +304,9 @@ class TkJobHandler(object):
         url = self.config[dashboard]['url']
 
         try:
+            # url + '/api/list-projects', timeout=self.timeout
             response = requests.get(
-                url + '/api/list-projects', timeout=self.timeout
+                url + '/api/projects/list', timeout=self.timeout
             )
             if response.status_code != 200:
                 logger.warning(
@@ -741,74 +744,3 @@ class TkJobHandler(object):
             dialog.destroy()
 
         return result
-
-
-if __name__ == "__main__":
-    import configargparse
-    import logging
-
-    logger = logging.getLogger(__name__)
-    parser = configargparse.ArgParser(
-        auto_env_var_prefix='',
-        default_config_files=[
-            '/etc/seamm/seamm.ini',
-            '~/.seamm/seamm.ini',
-        ],
-        description='Submit a SEAMM job to a dashboard'
-    )
-
-    parser.add_argument(
-        '--seamm-configfile',
-        is_config_file=True,
-        default=None,
-        help='a configuration file to override others'
-    )
-    parser.add_argument(
-        "-v",
-        "--verbose",
-        dest="verbose_count",
-        action="count",
-        default=0,
-        help="increases log verbosity for each occurence."
-    )
-
-    # Parse the configuration
-    args, unknown = parser.parse_known_args()
-
-    # Set up logging level to WARNING by default, going more verbose
-    # for each new -v, to INFO and then DEBUG and finally ALL with 3 -v's
-
-    numeric_level = max(3 - args.verbose_count, 0) * 10
-    logging.basicConfig(level=numeric_level)
-
-    # Initialize Tk
-    root = tk.Tk()
-    Pmw.initialise(root)
-
-    # s = ttk.Style()
-    # # print("Themes: " + str(s.theme_names()))
-    # rb = ttk.Progressbar()
-    # pprint.pprint(rb.configure())
-    # print('Label style: ' + rb['style'])
-    # class_ = rb.winfo_class()
-    # print('Label class: ' + class_)
-    # pprint.pprint(s.layout(class_))
-    # # pprint.pprint(s.element_options('Label'))
-    # # pprint.pprint(s.element_options('Label.border'))
-    # # s.configure("Border.TLabel", relief='ridge', anchor=tk.W, padding=5)
-
-    # Create the object
-    job_handler = TkJobHandler(root=root)
-
-    do_it = ttk.Button(
-        root, text='Submit...', command=job_handler.submit_with_dialog
-    )
-    do_it.grid(sticky=tk.EW)
-
-    status = ttk.Button(
-        root, text='Dashboards...', command=job_handler.display_dashboards
-    )
-    status.grid(row=1, sticky=tk.EW)
-
-    # Enter the event loop
-    root.mainloop()
